@@ -28,6 +28,7 @@
 #include <gsf/gsf-infile-msole.h>
 #include <gsf/gsf.h>
 #include <gsf/gsf-msole-impl.h>
+#include <glib/gi18n-lib.h>
 
 #include <string.h>
 #include <stdio.h>
@@ -52,6 +53,7 @@ typedef struct {
 	gboolean  is_directory;
 	GList	 *children;
 	unsigned char clsid[16];	/* 16 byte GUID used by some apps */
+	GDateTime *modtime;
 } MSOleDirent;
 
 typedef struct {
@@ -116,15 +118,15 @@ ole_seek_block (GsfInfileMSOle const *ole, guint32 block, gsf_off_t offset)
 }
 
 /**
- * ole_get_block :
+ * ole_get_block: (skip)
  * @ole: the infile
  * @block: block number
- * @buffer: optionally NULL
+ * @buffer: optionally %NULL
  *
  * Read a block of data from the underlying input.
  * Be really anal.
  *
- * Returns: pointer to the buffer or NULL if there is an error or 0 bytes are
+ * Returns: pointer to the buffer or %NULL if there is an error or 0 bytes are
  * requested.
  **/
 static guint8 const *
@@ -159,25 +161,22 @@ ole_make_bat (MSOleBAT const *metabat, size_t size_guess, guint32 block,
 	guint8 *used = (guint8*)g_alloca (1 + metabat->num_blocks / 8);
 	memset (used, 0, 1 + metabat->num_blocks / 8);
 
-	if (block < metabat->num_blocks)
-		do {
-			/* Catch cycles in the bat list */
-			if (used[block/8] & (1 << (block & 0x7)))
-				break;
-			used[block/8] |= 1 << (block & 0x7);
+	while (block < metabat->num_blocks) {
+		/* Catch cycles in the bat list */
+		if (used[block/8] & (1 << (block & 0x7)))
+			break;
+		used[block/8] |= 1 << (block & 0x7);
 
-			g_array_append_val (bat, block);
-			block = metabat->block [block];
-		} while (block < metabat->num_blocks);
-
-	res->block = NULL;
+		g_array_append_val (bat, block);
+		block = metabat->block [block];
+	}
 
 	res->num_blocks = bat->len;
 	res->block = (guint32 *) (gpointer) g_array_free (bat, FALSE);
 
 	if (block != BAT_MAGIC_END_OF_CHAIN) {
 		g_warning ("This OLE2 file is invalid.\n"
-			   "The Block Allocation Table for one of the streams had %x instead of a terminator (%x).\n"
+			   "The Block Allocation Table for one of the streams had 0x%08x instead of a terminator (0x%08x).\n"
 			   "We might still be able to extract some data, but you'll want to check the file.",
 			   block, BAT_MAGIC_END_OF_CHAIN);
 	}
@@ -277,6 +276,22 @@ ole_dirent_cmp (MSOleDirent const *a, MSOleDirent const *b)
 	return gsf_msole_sorting_key_cmp (a->key, b->key);
 }
 
+static GDateTime *
+datetime_from_filetime (guint64 ft)
+{
+	static const guint64 epoch = G_GINT64_CONSTANT (11644473600);
+	GTimeVal tv;
+	if (!ft)
+		return NULL;
+
+	/* ft is number of 100ns since Jan 1 1601 */
+
+	tv.tv_usec = (ft % 10000000u) / 10;
+	tv.tv_sec = (ft / 10000000u) - epoch;
+	return g_date_time_new_from_timeval_local (&tv);
+}
+
+
 /* Parse dirent number @entry and recursively handle its siblings and children.
  * parent is optional. */
 static MSOleDirent *
@@ -288,6 +303,7 @@ ole_dirent_new (GsfInfileMSOle *ole, guint32 entry, MSOleDirent *parent,
 	guint8 const *data;
 	guint8 type;
 	guint16 name_len;
+	guint64 ft;
 
 	if (entry >= DIRENT_MAGIC_END)
 		return NULL;
@@ -323,9 +339,12 @@ ole_dirent_new (GsfInfileMSOle *ole, guint32 entry, MSOleDirent *parent,
 	g_return_val_if_fail (type == DIRENT_TYPE_DIR || type == DIRENT_TYPE_ROOTDIR ||
 			      size <= (guint32)ole->input->size, NULL);
 
+	ft = GSF_LE_GET_GUINT64 (data + DIRENT_MODIFY_TIME);
+
 	dirent = g_new0 (MSOleDirent, 1);
 	dirent->index	     = entry;
 	dirent->size	     = size;
+	dirent->modtime = datetime_from_filetime (ft);
 	/* Store the class id which is 16 byte identifier used by some apps */
 	memcpy(dirent->clsid, data + DIRENT_CLSID, sizeof(dirent->clsid));
 
@@ -395,9 +414,15 @@ ole_dirent_free (MSOleDirent *dirent)
 	g_free (dirent->name);
 	gsf_msole_sorting_key_free (dirent->key);
 
-	for (tmp = dirent->children; tmp; tmp = tmp->next)
-		ole_dirent_free ((MSOleDirent *)tmp->data);
+	for (tmp = dirent->children; tmp; tmp = tmp->next) {
+		MSOleDirent *child = tmp->data;
+		ole_dirent_free (child);
+	}
 	g_list_free (dirent->children);
+
+	if (dirent->modtime)
+		g_date_time_unref (dirent->modtime);
+
 	g_free (dirent);
 }
 
@@ -443,7 +468,7 @@ ole_dup (GsfInfileMSOle const *src, GError **err)
 	if (input == NULL) {
 		if (err != NULL)
 			*err = g_error_new (gsf_input_error_id (), 0,
-					    "Failed to duplicate input stream");
+					    _("Failed to duplicate input stream"));
 		return NULL;
 	}
 
@@ -477,7 +502,7 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 	    0 != memcmp (header, signature, sizeof (signature))) {
 		if (err != NULL)
 			*err = g_error_new (gsf_input_error_id (), 0,
-				"No OLE2 signature");
+					    _("No OLE2 signature"));
 		return TRUE;
 	}
 
@@ -497,7 +522,7 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 	    (gsf_input_size (ole->input) >> bb_shift) < 1) {
 		if (err != NULL)
 			*err = g_error_new (gsf_input_error_id (), 0,
-				"Unreasonable block sizes");
+					    _("Unreasonable block sizes"));
 		return TRUE;
 	}
 
@@ -533,7 +558,7 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 			g_free (info);
 			if (err != NULL)
 				*err = g_error_new (gsf_input_error_id (), 0,
-						    "Insufficient memory");
+						    _("Insufficient memory"));
 			return TRUE;
 		}
 
@@ -592,7 +617,7 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 	if (fail) {
 		if (err != NULL)
 			*err = g_error_new (gsf_input_error_id (), 0,
-				"Inconsistent block allocation table");
+					    _("Inconsistent block allocation table"));
 		return TRUE;
 	}
 
@@ -600,7 +625,7 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 	if (ole_make_bat (&info->bb.bat, 0, dirent_start, &ole->bat)) {
 		if (err != NULL)
 			*err = g_error_new (gsf_input_error_id (), 0,
-				"Problems making block allocation table");
+					    _("Problems making block allocation table"));
 		return TRUE;
 	}
 
@@ -612,10 +637,16 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 	if (ole->dirent == NULL) {
 		if (err != NULL)
 			*err = g_error_new (gsf_input_error_id (), 0,
-				"Problems reading directory");
+					    _("Problems reading directory"));
 		return TRUE;
 	}
 
+	/*
+	 * The spec says to ignore modtime for root object.  That doesn't
+	 * keep files from actually have a modtime there.
+	 */
+	gsf_input_set_modtime (GSF_INPUT (ole), ole->dirent->modtime);
+
 	return FALSE;
 }
 
@@ -671,17 +702,21 @@ gsf_infile_msole_read (GsfInput *input, size_t num_bytes, guint8 *buffer)
 	last_block = OLE_BIG_BLOCK (input->cur_offset + num_bytes - 1, ole);
 	offset = input->cur_offset & ole->info->bb.filter;
 
-	/* optimization : are all the raw blocks contiguous */
+	if (last_block >= ole->bat.num_blocks)
+		return NULL;
+
+	/* Optimization: are all the raw blocks contiguous?  */
 	i = first_block;
-	raw_block = ole->bat.block [i];
+	raw_block = ole->bat.block[i];
+	if (FALSE && first_block != last_block)
+		g_printerr ("Check if %ld-%ld of %d are contiguous.\n",
+			    first_block, last_block,
+			    ole->bat.num_blocks);
 	while (++i <= last_block && ++raw_block == ole->bat.block [i])
 		;
 	if (i > last_block) {
-		/* optimization don't seek if we don't need to */
-		if (ole->cur_block != first_block) {
-			if (!ole_seek_block (ole, ole->bat.block [first_block], offset))
-				return NULL;
-		}
+		if (!ole_seek_block (ole, ole->bat.block [first_block], offset))
+			return NULL;
 		ole->cur_block = last_block;
 		return gsf_input_read (ole->input, num_bytes, buffer);
 	}
@@ -740,6 +775,7 @@ gsf_infile_msole_new_child (GsfInfileMSOle *parent,
 
 	child->dirent = dirent;
 	gsf_input_set_size (GSF_INPUT (child), (gsf_off_t) dirent->size);
+	gsf_input_set_modtime (GSF_INPUT (child), dirent->modtime);
 
 	/* The root dirent defines the small block file */
 	if (dirent->index != 0) {
@@ -764,7 +800,7 @@ gsf_infile_msole_new_child (GsfInfileMSOle *parent,
 		if (!sb_file) {
 			if (err != NULL)
 				*err = g_error_new (gsf_input_error_id (), 0,
-						    "Failed to access child");
+						    _("Failed to access child"));
 			g_object_unref (child);
 			return NULL;
 		}
@@ -795,7 +831,9 @@ gsf_infile_msole_new_child (GsfInfileMSOle *parent,
 						    child->stream.buf + (i << info->sb.shift))) == NULL) {
 
 				g_warning ("failure reading block %d for '%s'", i, dirent->name);
-				if (err) *err = g_error_new (gsf_input_error_id (), 0, "failure reading block");
+				if (err) *err = g_error_new
+						 (gsf_input_error_id (), 0,
+						  _("failure reading block"));
 				g_object_unref (child);
 				return NULL;
 			}
