@@ -24,6 +24,7 @@
 #include <gsf/gsf-infile-zip.h>
 #include <gsf/gsf.h>
 #include <gsf/gsf-zip-impl.h>
+#include <glib/gi18n-lib.h>
 
 #include <string.h>
 #include <zlib.h>
@@ -76,6 +77,26 @@ typedef struct {
 #define GSF_INFILE_ZIP_CLASS(k)    (G_TYPE_CHECK_CLASS_CAST ((k), GSF_INFILE_ZIP_TYPE, GsfInfileZipClass))
 #define GSF_IS_INFILE_ZIP_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GSF_INFILE_ZIP_TYPE))
 
+
+static GDateTime *
+zip_make_modtime (guint32 dostime)
+{
+	if (dostime == 0)
+		return NULL;
+	else {
+		gint year = (dostime >> 25) + 1980;
+		gint month = (dostime >> 21) & 0x0f;
+		gint day = (dostime >> 16) & 0x1f;
+		gint hour = (dostime >> 11) & 0x0f;
+		gint minute = (dostime >> 5) & 0x3f;
+		gint second = (dostime & 0x1f) * 2;
+		GDateTime *modtime =
+			g_date_time_new_utc (year, month, day,
+					     hour, minute, second);
+		return modtime;
+	}
+}
+
 static GsfZipVDir *
 vdir_child_by_name (GsfZipVDir *vdir, char const *name)
 {
@@ -189,7 +210,7 @@ zip_dirent_new_in (GsfInfileZip *zip, gsf_off_t *offset)
 	GsfZipDirent *dirent;
 	guint8 const *data;
 	guint16 name_len, extras_len, comment_len, compr_method, flags;
-	guint32 crc32, csize, usize, off;
+	guint32 dostime, crc32, csize, usize, off;
 	gchar *name;
 
 	/* Read data and check the header */
@@ -205,6 +226,7 @@ zip_dirent_new_in (GsfInfileZip *zip, gsf_off_t *offset)
 
 	flags =         GSF_LE_GET_GUINT32 (data + ZIP_DIRENT_FLAGS);
 	compr_method =  GSF_LE_GET_GUINT16 (data + ZIP_DIRENT_COMPR_METHOD);
+	dostime =       GSF_LE_GET_GUINT32 (data + ZIP_DIRENT_DOSTIME);
 	crc32 =         GSF_LE_GET_GUINT32 (data + ZIP_DIRENT_CRC32);
 	csize =         GSF_LE_GET_GUINT32 (data + ZIP_DIRENT_CSIZE);
 	usize =         GSF_LE_GET_GUINT32 (data + ZIP_DIRENT_USIZE);
@@ -226,6 +248,7 @@ zip_dirent_new_in (GsfInfileZip *zip, gsf_off_t *offset)
 	dirent->csize =         csize;
 	dirent->usize =         usize;
 	dirent->offset =        off;
+	dirent->dostime =       dostime;
 #if 0
 	g_print ("%s = 0x%x @ %" GSF_OFF_T_FORMAT "\n", name, off, *offset);
 #endif
@@ -304,14 +327,14 @@ zip_read_dirents (GsfInfileZip *zip)
 	offset = zip_find_trailer (zip);
 	if (offset < 0) {
 		zip->err = g_error_new (gsf_input_error_id (), 0,
-					"No Zip trailer");
+					_("No Zip trailer"));
 		return TRUE;
 	}
 
 	if (gsf_input_seek (zip->source, offset, G_SEEK_SET) ||
 	    NULL == (trailer = gsf_input_read (zip->source, ZIP_TRAILER_SIZE, NULL))) {
 		zip->err = g_error_new (gsf_input_error_id (), 0,
-					"Error reading Zip signature");
+					_("Error reading Zip signature"));
 		return TRUE;
 	}
 
@@ -332,7 +355,7 @@ zip_read_dirents (GsfInfileZip *zip)
 		d = zip_dirent_new_in (zip, &offset);
 		if (d == NULL) {
 			zip->err = g_error_new (gsf_input_error_id (), 0,
-						"Error reading zip dirent");
+						_("Error reading zip dirent"));
 			return TRUE;
 		}
 
@@ -386,7 +409,7 @@ zip_child_init (GsfInfileZip *child, GError **errmsg)
 		{ 'P', 'K', 0x03, 0x04 };
 	guint8 const *data = NULL;
 	guint16 name_len, extras_len;
-	char *err = NULL;
+	const char *err = NULL;
 
 	GsfZipDirent *dirent = child->vdir->dirent;
 
@@ -395,13 +418,11 @@ zip_child_init (GsfInfileZip *child, GError **errmsg)
 	 **/
 
 	if (gsf_input_seek (child->source, (gsf_off_t) dirent->offset, G_SEEK_SET))
-		err = g_strdup_printf ("Error seeking to zip header @ %" GSF_OFF_T_FORMAT,
-				       dirent->offset);
+		err = _("Error seeking to zip header");
 	else if (NULL == (data = gsf_input_read (child->source, ZIP_FILE_HEADER_SIZE, NULL)))
 		err = g_strdup_printf ("Error reading %d bytes in zip header", ZIP_FILE_HEADER_SIZE);
 	else if (0 != memcmp (data, header_signature, sizeof (header_signature))) {
-		err = g_strdup_printf ("Error incorrect zip header @ %" GSF_OFF_T_FORMAT,
-				       dirent->offset);
+		err = _("Error incorrect zip header");
 		g_print ("Header is :\n");
 		gsf_mem_dump (data, sizeof (header_signature));
 		g_print ("Header should be :\n");
@@ -411,7 +432,6 @@ zip_child_init (GsfInfileZip *child, GError **errmsg)
 	if (NULL != err) {
 		if (errmsg != NULL)
 			*errmsg = g_error_new_literal (gsf_input_error_id (), 0, err);
-		g_free (err);
 		return TRUE;
 	}
 
@@ -435,7 +455,7 @@ zip_child_init (GsfInfileZip *child, GError **errmsg)
 		if (err != Z_OK) {
 			if (errmsg != NULL)
 				*errmsg = g_error_new (gsf_input_error_id (), 0,
-						       "problem uncompressing stream");
+						       _("problem uncompressing stream"));
 			return TRUE;
 		}
 	}
@@ -613,6 +633,13 @@ gsf_infile_zip_new_child (GsfInfileZip *parent, GsfZipVDir *vdir, GError **err)
 	if (dirent) {
 		gsf_input_set_size (GSF_INPUT (child),
 				    (gsf_off_t) dirent->usize);
+
+		if (dirent->dostime) {
+			GDateTime *modtime = zip_make_modtime (dirent->dostime);
+			gsf_input_set_modtime (GSF_INPUT (child), modtime);
+			g_date_time_unref (modtime);
+		}
+
 		if (zip_child_init (child, err) != FALSE) {
 			g_object_unref (child);
 			return NULL;
@@ -817,8 +844,8 @@ gsf_infile_zip_class_init (GObjectClass *gobject_class)
 		(gobject_class,
 		 PROP_SOURCE,
 		 g_param_spec_object ("source",
-				      "Source",
-				      "The archive being interpreted.",
+				      _("Source"),
+				      _("The archive being interpreted"),
 				      GSF_INPUT_TYPE,
 				      GSF_PARAM_STATIC |
 				      G_PARAM_READWRITE |
@@ -833,12 +860,13 @@ gsf_infile_zip_class_init (GObjectClass *gobject_class)
 		(gobject_class,
 		 PROP_COMPRESSION_LEVEL,
 		 g_param_spec_int ("compression-level",
-				   "Compression Level",
-				   "The level of compression used, zero meaning none.",
+				   _("Compression Level"),
+				   _("The level of compression used, zero meaning none"),
 				   0, 10,
 				   0,
 				   GSF_PARAM_STATIC |
 				   G_PARAM_READABLE));
+
 	g_object_class_install_property
 		(gobject_class,
 		 PROP_INTERNAL_PARENT,
@@ -858,7 +886,7 @@ GSF_CLASS (GsfInfileZip, gsf_infile_zip,
 /**
  * gsf_infile_zip_new:
  * @source: A base #GsfInput
- * @err: A #GError, optionally %null
+ * @err: (allow-none): place to store a #GError if anything goes wrong
  *
  * Opens the root directory of a Zip file.
  * <note>This adds a reference to @source.</note>
