@@ -1103,7 +1103,7 @@ msole_prop_cmp (gconstpointer a, gconstpointer b)
  *
  * Since: 1.14.24
  *
- * Returns: GError which the caller must free on error.
+ * Returns: (transfer full): A #GError if there was an error.
  **/
 GError *
 gsf_doc_meta_data_read_from_msole (GsfDocMetaData *accum, GsfInput *in)
@@ -1124,7 +1124,7 @@ gsf_doc_meta_data_read_from_msole (GsfDocMetaData *accum, GsfInput *in)
 	data = gsf_input_read (in, 28, NULL);
 	if (NULL == data)
 		return g_error_new (gsf_input_error_id (), 0,
-			"Unable to read MS property stream header");
+				    _("Unable to read MS property stream header"));
 
 	d ({g_print ("===================================\n"
 		   "header class id ==\n");
@@ -1151,7 +1151,7 @@ gsf_doc_meta_data_read_from_msole (GsfDocMetaData *accum, GsfInput *in)
 	    || num_sections > gsf_input_size(in) / 20
 	    || num_sections > 100) /* arbitrary sanity check */
 		return g_error_new (gsf_input_error_id (), 0,
-			"Invalid MS property stream header");
+				    _("Invalid MS property stream header"));
 
 	/* extract the section info */
 	/*
@@ -1168,7 +1168,7 @@ gsf_doc_meta_data_read_from_msole (GsfDocMetaData *accum, GsfInput *in)
 		data = gsf_input_read (in, 20, NULL);
 		if (NULL == data)
 			return g_error_new (gsf_input_error_id (), 0,
-				"Unable to read MS property stream header");
+					    _("Unable to read MS property stream header"));
 		if (!memcmp (data, component_guid, sizeof (component_guid)))
 			sections [i].type = COMPONENT_PROP;
 		else if (!memcmp (data, document_guid, sizeof (document_guid)))
@@ -1198,7 +1198,7 @@ gsf_doc_meta_data_read_from_msole (GsfDocMetaData *accum, GsfInput *in)
 		if (gsf_input_seek (in, sections[i].offset, G_SEEK_SET) ||
 		    NULL == (data = gsf_input_read (in, 8, NULL)))
 			return g_error_new (gsf_input_error_id (), 0,
-				"Invalid MS property section");
+					    _("Invalid MS property section"));
 
 		sections[i].iconv_handle = (GIConv)-1;
 		sections[i].char_size    = 1;
@@ -1217,11 +1217,11 @@ gsf_doc_meta_data_read_from_msole (GsfDocMetaData *accum, GsfInput *in)
 			continue;
 		if (sections[i].num_props > gsf_input_remaining(in) / 8)
 			return g_error_new (gsf_input_error_id (), 0,
-				"Invalid MS property stream header or file truncated");
+					    _("Invalid MS property stream header or file truncated"));
 
 		if (sections[i].offset + sections[i].size > gsf_input_size(in))
 			return g_error_new (gsf_input_error_id (), 0,
-				"Invalid MS property stream header or file truncated");
+					    _("Invalid MS property stream header or file truncated"));
 
 		/*
 		 * Get and save all the Property ID/Offset pairs.
@@ -1236,7 +1236,7 @@ gsf_doc_meta_data_read_from_msole (GsfDocMetaData *accum, GsfInput *in)
 			if (NULL == (data = gsf_input_read (in, 8, NULL))) {
 				g_free (props);
 				return g_error_new (gsf_input_error_id (), 0,
-					"Invalid MS property section");
+						    _("Invalid MS property section"));
 			}
 
 			props[j].id = GSF_LE_GET_GUINT32 (data);
@@ -1260,7 +1260,7 @@ gsf_doc_meta_data_read_from_msole (GsfDocMetaData *accum, GsfInput *in)
 			if (props[j].offset < 0 || props[j].offset + 4 > end) {
 				g_free (props);
 				return g_error_new (gsf_input_error_id (), 0,
-					"Invalid MS property section");
+						    _("Invalid MS property section"));
 			}
 		}
 
@@ -1319,7 +1319,7 @@ gsf_doc_meta_data_read_from_msole (GsfDocMetaData *accum, GsfInput *in)
  *
  * Deprecated: 1.14.24, use gsf_doc_meta_data_read_from_msole
  *
- * Returns: GError which the caller must free on error.
+ * Returns: (transfer full): A #GError if there was an error.
  **/
 GError *
 gsf_msole_metadata_read	(GsfInput *in, GsfDocMetaData *accum)
@@ -1967,10 +1967,10 @@ static struct {
 
 /**
  * gsf_msole_lid_for_language:
- * @lang:
+ * @lang: (allow-none): Language id, i.e., locale name.
  *
  * Returns: the LID (Language Identifier) for the input language.
- * 	If lang is %null, return 0x0400 ("-none-"), and not 0x0000 ("no proofing")
+ * 	If lang is %NULL, return 0x0400 ("-none-"), and not 0x0000 ("no proofing")
  **/
 guint
 gsf_msole_lid_for_language (char const *lang)
@@ -1992,11 +1992,11 @@ gsf_msole_lid_for_language (char const *lang)
 
 /**
  * gsf_msole_language_for_lid:
- * @lid:
+ * @lid: numerical language id
  *
- * Returns: the xx_YY style string (can be just xx or xxx) for the given LID.
- * 	Return value must not be freed. If the LID is not found, is set to
- * 	0x0400, or is set to 0x0000, will return "-none-"
+ * Returns: (transfer none): the xx_YY style string (can be just xx or
+ * xxx) for the given LID.  If the LID is not found, is set to 0x0400,
+ * or is set to 0x0000, will return "-none-"
  **/
 char const *
 gsf_msole_language_for_lid (guint lid)
@@ -2012,6 +2012,7 @@ gsf_msole_language_for_lid (guint lid)
 
 /**
  * gsf_msole_locale_to_lid:
+ * @codepage: character code page.
  *
  * Convert the the codepage into an applicable LID
  **/
@@ -2057,7 +2058,7 @@ gsf_msole_codepage_to_lid (int codepage)
 
 /**
  * gsf_msole_lid_to_codepage:
- * @lid:
+ * @lid: numerical language id
  *
  * Returns: our best guess at the codepage for the given language id
  **/
@@ -2258,10 +2259,10 @@ gsf_msole_lid_to_codepage (guint lid)
 
 /**
  * gsf_msole_lid_to_codepage_str:
- * @lid:
+ * @lid: numerical language id
  *
- * Returns: the Iconv codepage string for the given LID.
- * 	Return value must be g_free ()'d
+ * Returns: (transfer full): the Iconv codepage string for the given
+ * LID.
  **/
 gchar *
 gsf_msole_lid_to_codepage_str (guint lid)
