@@ -126,7 +126,7 @@ gsf_open_pkg_rel_is_extern (GsfOpenPkgRel const *rel)
  * gsf_open_pkg_rel_get_target:
  * @rel: #GsfOpenPkgRel
  *
- * Returns: const pointer to @rel's target.
+ * Returns: (transfer none): The target of @rel.
  **/
 char const *
 gsf_open_pkg_rel_get_target (GsfOpenPkgRel const *rel)
@@ -139,7 +139,7 @@ gsf_open_pkg_rel_get_target (GsfOpenPkgRel const *rel)
  * gsf_open_pkg_rel_get_type:
  * @rel: #GsfOpenPkgRel
  *
- * Returns: const pointer to @rel's type.
+ * Returns: (transfer none): The type of @rel.
  **/
 char const *
 gsf_open_pkg_rel_get_type (GsfOpenPkgRel const *rel)
@@ -392,7 +392,7 @@ gsf_open_pkg_open_rel_by_type (GsfInput *opkg, char const *type, GError **err)
  *
  * Convenience function to parse a related part.
  *
- * Returns: NULL on success or a GError which callerss need to free on failure.
+ * Returns: (transfer full): %NULL on success or a #GError on failure.
  **/
 GError *
 gsf_open_pkg_parse_rel_by_id (GsfXMLIn *xin, char const *id,
@@ -725,14 +725,25 @@ gsf_outfile_open_pkg_class_init (GObjectClass *gobject_class)
 	parent_class = g_type_class_peek_parent (gobject_class);
 
 	g_object_class_install_property (gobject_class, PROP_SINK,
-		 g_param_spec_object ("sink", "Sink", "The GsfOutput that stores the Open Package content.",
-			GSF_OUTFILE_TYPE, GSF_PARAM_STATIC | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+		 g_param_spec_object ("sink",
+				      _("Sink"),
+				      _("The GsfOutput that stores the Open Package content"),
+				      GSF_OUTFILE_TYPE,
+				      GSF_PARAM_STATIC | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
 	g_object_class_install_property (gobject_class, PROP_CONTENT_TYPE,
-		 g_param_spec_string ("content-type", "ContentType", "The ContentType stored in the root [Content_Types].xml file.",
-			"", GSF_PARAM_STATIC | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+		 g_param_spec_string ("content-type",
+				      _("Content type"),
+				      _("The content type stored in the root [Content_Types].xml file"),
+				      "",
+				      GSF_PARAM_STATIC | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
 	g_object_class_install_property (gobject_class, PROP_IS_DIR,
-		 g_param_spec_boolean ("is-dir", "IsDir", "Can the outfile have children.",
-			FALSE, GSF_PARAM_STATIC | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+		 g_param_spec_boolean ("is-dir",
+				       _("Is Directory"),
+				       _("Can the outfile have children"),
+				       FALSE,
+				       GSF_PARAM_STATIC | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 GSF_CLASS (GsfOutfileOpenPkg, gsf_outfile_open_pkg,
@@ -813,8 +824,8 @@ gsf_outfile_open_pkg_create_rel (GsfOutfileOpenPkg *parent,
  *
  * Create a relationship between @child and @parent of @type.
  *
- * Returns: the relID which the caller does not own but will live as long as
- * 	@parent.
+ * Returns: (transfer none): the relID which the caller does not own
+ * 	but will live as long as @parent.
  **/
 char const *
 gsf_outfile_open_pkg_relate (GsfOutfileOpenPkg *child,
@@ -889,8 +900,9 @@ gsf_outfile_open_pkg_add_rel (GsfOutfile *dir,
  *
  * Add an external relation to @parent.
  *
- * Returns: (transfer none): The id of the relation.  The string is managed by the parent and
- * 	should not be changed or freed by the caller.
+ * Returns: (transfer none): The id of the relation.  The string is
+ * managed by the parent and should not be changed or freed by the
+ * caller.
  **/
 char const *
 gsf_outfile_open_pkg_add_extern_rel (GsfOutfileOpenPkg *parent,
