@@ -23,6 +23,7 @@
 #include <gsf/gsf-outfile-zip.h>
 #include <gsf/gsf.h>
 #include <gsf/gsf-zip-impl.h>
+#include <glib/gi18n-lib.h>
 
 #include <string.h>
 #include <time.h>
@@ -131,6 +132,12 @@ gsf_outfile_zip_constructor (GType                  type,
 		gsf_output_set_container (GSF_OUTPUT (zip), NULL);
 	}
 
+	if (!gsf_output_get_modtime (GSF_OUTPUT (zip))) {
+		GDateTime *modtime = g_date_time_new_now_utc ();
+		gsf_output_set_modtime (GSF_OUTPUT (zip), modtime);
+		g_date_time_unref (modtime);
+	}
+
 	return (GObject *)zip;
 }
 
@@ -258,17 +265,33 @@ stream_name_build (GsfOutfileZip *zip)
 }
 
 static guint32
-zip_time_make (time_t t)
+zip_time_make (GDateTime *modtime)
 {
-	struct tm *localnow = localtime (&t);
+	gint year, month, day, hour, minute, second;
 	guint32 ztime;
 
-	ztime = (localnow->tm_year - 80) & 0x7f;
-	ztime = (ztime << 4) | ((localnow->tm_mon + 1)  & 0x0f);
-	ztime = (ztime << 5) | (localnow->tm_mday & 0x1f);
-	ztime = (ztime << 5) | (localnow->tm_hour & 0x1f);
-	ztime = (ztime << 6) | (localnow->tm_min  & 0x3f);
-	ztime = (ztime << 5) | ((localnow->tm_sec / 2) & 0x1f);
+	if (!modtime)
+		modtime = g_date_time_new_now_utc ();
+	else
+		g_date_time_ref (modtime);
+
+	g_date_time_get_ymd (modtime, &year, &month, &day);
+	hour = g_date_time_get_hour (modtime);
+	minute = g_date_time_get_minute (modtime);
+	second = g_date_time_get_second (modtime);
+
+	if (year < 1980 || year > 1980 + 0x7f)
+		ztime = 0;
+	else {
+		ztime = (year - 1980) & 0x7f;
+		ztime = (ztime << 4) | (month  & 0x0f);
+		ztime = (ztime << 5) | (day & 0x1f);
+		ztime = (ztime << 5) | (hour & 0x1f);
+		ztime = (ztime << 6) | (minute & 0x3f);
+		ztime = (ztime << 5) | ((second / 2) & 0x1f);
+	}
+
+	g_date_time_unref (modtime);
 
 	return ztime;
 }
@@ -288,7 +311,7 @@ zip_dirent_new_out (GsfOutfileZip *zip)
 	GsfZipDirent *dirent = gsf_zip_dirent_new ();
 	dirent->name = stream_name_build (zip);
 	dirent->compr_method = zip->compression_method;
-	dirent->dostime = zip_time_make (time (NULL));
+	dirent->dostime = zip_time_make (gsf_output_get_modtime (GSF_OUTPUT (zip)));
 	zip_dirent_update_flags (dirent);
 	return dirent;
 }
@@ -585,6 +608,12 @@ gsf_outfile_zip_new_child (GsfOutfile *parent,
 					      &params, &n_params,
 					      first_property_name,
 					      args);
+	if (!gsf_property_settings_find ("modtime", params, n_params))
+		gsf_property_settings_collect (GSF_OUTFILE_ZIP_TYPE,
+					       &params, &n_params,
+					       "modtime", gsf_output_get_modtime (GSF_OUTPUT (parent)),
+					       NULL);
+
 	child = (GsfOutfileZip *)g_object_newv (GSF_OUTFILE_ZIP_TYPE,
 						n_params,
 						params);
@@ -702,8 +731,9 @@ gsf_outfile_zip_class_init (GObjectClass *gobject_class)
 	g_object_class_install_property
 		(gobject_class,
 		 PROP_SINK,
-		 g_param_spec_object ("sink", "Sink",
-				      "Where the archive is written.",
+		 g_param_spec_object ("sink",
+				      _("Sink"),
+				      _("Where the archive is written"),
 				      GSF_OUTPUT_TYPE,
 				      GSF_PARAM_STATIC |
 				      G_PARAM_READWRITE |
@@ -711,8 +741,9 @@ gsf_outfile_zip_class_init (GObjectClass *gobject_class)
 	g_object_class_install_property
 		(gobject_class,
 		 PROP_ENTRY_NAME,
-		 g_param_spec_string ("entry-name", "Entry Name",
-				      "The filename of this member in the archive without path.",
+		 g_param_spec_string ("entry-name",
+				      _("Entry Name"),
+				      _("The filename of this member in the archive without path"),
 				      NULL,
 				      GSF_PARAM_STATIC |
 				      G_PARAM_READWRITE |
@@ -721,8 +752,8 @@ gsf_outfile_zip_class_init (GObjectClass *gobject_class)
 		(gobject_class,
 		 PROP_COMPRESSION_LEVEL,
 		 g_param_spec_int ("compression-level",
-				   "Compression Level",
-				   "The level of compression used, zero meaning none.",
+				   _("Compression Level"),
+				   _("The level of compression used, zero meaning none"),
 				   0, 10,
 				   GSF_ZIP_DEFLATED,
 				   GSF_PARAM_STATIC |
